;***************************************************************************
;* TINYFOX 80m
;* 
;* File Name				: "Tinyfox52.asm"
;* Title						: TINYFOX 80m
;* Author					: Matthias Kuehlewein, DL3SDO
;* Date						: 19.5.2020
;* Version					: 5.2
;* Target MCU				: ATtiny25/ATtiny45
;* Fuses (RC mode)		: Default, except BODLEVEL1 = 0 => VBOT = 2.7V
;*                        Fuse Extended Byte: 0xFF
;*                        Fuse High Byte: 0xDD
;*                        Fuse Low Byte: 0x62
;* Fuses (QUARTZ mode)	: Default, except BODLEVEL1 = 0 => VBOT = 2.7V
;*                                 and CKDIV8 = 1
;*                                 and CKSEL[3:0] = [0000] => External Clock
;*                        Fuse Extended Byte: 0xFF
;*                        Fuse High Byte: 0xDD
;*                        Fuse Low Byte: 0xE0
;* Compiler					: AVRASM2 (2.2.0) contained in Atmel Studio 7
;* Editor					: use tabwidth 3
;*
;* DESCRIPTION
;* This is the software for the Atmel ATtiny25/45 on the Tinyfox Foxoring
;* transmitter. 
;* This software version runs internally with 8MHz / 8 = 1MHz
;* or clocked by the 3,579545MHz clock out of the 74HC00 osc. gate
;*
;*
;*    Copyright (C) 2005-2020  Matthias Kuehlewein, DL3SDO
;*
;*    This program is free software: you can redistribute it and/or modify
;*    it under the terms of the GNU General Public License as published by
;*    the Free Software Foundation, either version 3 of the License, or
;*    (at your option) any later version.
;*
;*    This program is distributed in the hope that it will be useful,
;*    but WITHOUT ANY WARRANTY; without even the implied warranty of
;*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*    GNU General Public License for more details.
;*
;*    You should have received a copy of the GNU General Public License
;*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;*
;***************************************************************************

;***************************************************************************
; NOTES
; 
; some keywords are used within the comments:
; TBD				To be done			Something is to be checked or modified
; TEST			Test					Code for test and development purposes
; OPT: yy		Optimization		Code was optimized, take care when changing
; !DEP_x: yy!	Dependency			There is one or more dependencies between code parts
;											x is the number, so !DEP_1!, !DEP_2! etc. are used
;											yy may contain additional (explanatory) text
; current highest DEP number: 7
;
;***************************************************************************


;***************************************************************************
; include one of the two include files below to select the C type
; if the -D MCU=XX option is not used by the batch file
;***************************************************************************
#ifndef MCU
.INCLUDE "tn45def.inc"						; include made by ATMEL
;.INCLUDE "tn25def.inc"						; include made by ATMEL

;***************************************************************************
;* or let the batch file make the choice
;***************************************************************************
#else
#if MCU==45
.INCLUDE "tn45def.inc"						; include made by ATMEL
#elif MCU==25
.INCLUDE "tn25def.inc"						; include made by ATMEL
#else
#error "Unknown device"
#endif
#endif

;***************************************************************************
; selection of quartz mode by batch file
;***************************************************************************
#ifdef QUARTZ
#define QuartzClock							; Use 74HC00 oscillator output as clock source for the ATtiny
#endif

;The X register is not used by this software.
;To avoid unnecessary warnings, X is undefined below.
.UNDEF XH
.UNDEF XL

;****************************************************************************
;*
;* Definitions and Settings
;*
;****************************************************************************
;****************************************************************************
; Program switches
;****************************************************************************
; Configuration only with LED or also with keyed TX
;#define LedOnlyConfig

; Use settings and code for test
;#define TEST

;****************************************************************************
; Constant and Parameter Settings
;****************************************************************************
; Programm Version
#ifdef __ATtiny45__
.EQU Version = 0x52
#endif
#ifdef __ATtiny25__
.EQU Version = 0x52
#endif

; Layout Version
.EQU LayoutVersion = 0x01

; Magic Number for OSCCAL, currently not used
.EQU OscCalMagic = 0x6b

; Default (initial) Key
#ifndef TEST
.EQU InitKey = 0								; 'a'
;.EQU InitKey = (StringCharsStart-PatternStart)*2 ;StringChars				; moe
#else
.EQU InitKey = (StringCharsStart-PatternStart)*2+4*4 ;StringChars				; TEST
#endif

.EQU MOEKey = (StringCharsStart-PatternStart)*2		; for key configuration 

; Macros to convert chars ('a'-'z')/digits ('0'-'9') to KeyPattern references
#define CK(X) ((X)-97)						; for lowercase letters
#define DK(X) ((X)-48+26)					; for digits

; Default Keying Speed
#ifndef TEST
.EQU InitSpeed = 5							; 44 BpM, see table below
#else
.EQU InitSpeed = 6							; TEST
#endif

; Keying Speed (smaller values => faster), duration for one dot
; in multiples of 0.017152s

; Keying speed = 60/(10*DotTimes*0,017152)
; Speed value	=> 	DotTimes			=>		Keying speed
;		0			=>			13				=>				26,9
;		1			=>			12				=>				29,2
;		2			=>			11				=>				31,8
;		3			=>			10				=>				35
;		4			=>			9				=>				38,9
;		5			=>			8				=>				43,7
;		6			=>			7				=>				50
;		7			=>			6				=>				58,3
;		8			=>			5				=>				70
;		9			=>			4				=>				87,5


; Default VBat configuration value
; 0: no VBat measurement
; 1: VBat measurement and output of value twice
; 2: VBat measurement and compare with VBat1 (= battery)
; 3: VBat measurement and compare with VBat2 (= rechargeable battery)
#ifndef TEST
.EQU InitVBat = 2								; V5.2: changed from 1 to 2
#else
.EQU InitVBat = 1								; TEST
#endif

; Default Callsign Speed
#ifndef TEST
.EQU InitCallsignSpeed = 5					; 44 BpM, see table above
#else
.EQU InitCallsignSpeed = 6					; TEST
#endif

; Default Callsign Mode
; 0: no callsign transmission
; 1: transmit only callsign
; 2: transmit callsign every 5min		; !DEP_1!
; 3: transmit callsign every 10min
; 4: transmit callsign every 15min
; 5: transmit callsign every 20min
#ifndef TEST
.EQU InitCallsignMode = 0
#else
.EQU InitCallsignMode = 1					; TEST
#endif


; Default Delay
; InitDelayHi between 0 and 5, unit is 10hrs
; InitDelayLo between 0 and 19, unit is 30min
; => Delay can be between 0 and 59,5h
; Delay 30min can not be configured
#ifndef TEST
.EQU InitDelayHi = 0
.EQU InitDelayLo = 0
#else
.EQU InitDelayHi = 0							; TEST
.EQU InitDelayLo = 0							; TEST
#endif

; Default Delay Mode
; 0: no Delay applied
; 1: apply Delay if Pin pulled low
; 2: always apply Delay
#ifndef TEST
.EQU InitDelayMode = 0
#else
.EQU InitDelayMode = 0						; TEST
#endif


; Default transmission cycle
; 0: continuous transmission
; 1: 1min interval (sprint)
; 2: 5min interval
#ifndef TEST
.EQU InitTransCycle = 0
#else
.EQU InitTransCycle = 0						; TEST
#endif

; Default position in transmission cycle
; values between 1 and 5
.EQU InitTransCycPos = 1

; Default value for TX configuration
; values between 0 and 1
; 0: only LED indication
; 1: LED and TX indication
.EQU InitConfigWithTX = 1


; The formula 12s*clockfreq./(256*OCR0A) gives the value for
; the 12s duration
; ExtendISR12s, ExtendISR1min and ExtendISR5min are used
; for finetuning:
; ExtendISR12s increases value by (0,2 * ExtendISR12s) 				(5*12s	=	1min => 1/5 		= 0,2)
; ExtendISR1min increases value by (0,04 * ExtendISR1min)			(5*1min	=	5min => 1/(5*5)	= 0,04)
; ExtendISR5min increases value by (0,0066666.. * ExtendISR5min)	(6*5min	= 30min => 1/(5*5*6)	= 0,0066666..)

#ifndef QuartzClock
; The 3 digits of the duration for 12s (=699dec), see TIM0_COMPA ISR
; (for a (8MHz/8) RC oscillator with a 256*67 divider)
; exact value = 12s*1MHz/(67*256) = 699,626865671641791
.EQU Digit12sFirst = 6
.EQU Digit12sMiddle = 9
.EQU Digit12sLast = 9
; how often extend the 12s interval duration by one
.EQU ExtendISR12s = 3
; how often extend the 1min interval duration by one
.EQU ExtendISR1min = 0						; RC is not precise enough to correct more precise
; how often extend the 5min interval duration by one
.EQU ExtendISR5min = 0						; RC is not precise enough to correct more precise

#else

; The 3 digits of the duration for 12s (=699dec), see TIM0_COMPA ISR
; (for a 3,579545MHz quartz with a 256*240 divider)
; exact value = 12s*3,579545MHz/(240*256) = 699,1298828125
.EQU Digit12sFirst = 6
.EQU Digit12sMiddle = 9
.EQU Digit12sLast = 9
; how often extend the 12s interval duration by one
.EQU ExtendISR12s = 0
; how often extend the 1min interval duration by one
.EQU ExtendISR1min = 3
; how often extend the 5min interval duration by one
.EQU ExtendISR5min = 1
; this results in a value of 699 + 0*0,2 + 3*0,04 + 1*0,006667 = 699,126667
#endif

; Values for some quartzes:
;                  3,500	 3,510	 3,530	 3,540	 3,550	 3,555	 3,560	 3,570	3,579545		 3,600
; Digit12sFirst:		6			6			6			6			6			6			6			6			6				7
; Digit12sMiddle:		8			8			8			9			9			9			9			9			9				0
; Digit12sLast:		3			5			9			1			3			4			5			7			9				3
;
; ExtendISR12s:		2			2			2			2			1			1			1			1			0				0
; ExtendISR1min:		4			3			1			0			3			3			2			1			3				3
; ExtendISR5min:		5			4			1			0			5			2			4			3			1				0


; Note: although the transmission intervals of 12s, 1min, 5min etc. can be met by using the values above
; for other quartzes than the 3,579545MHz one, the keying speed will be different.
; If the keying speed shall also remain the same there is a way to achieve it:
; In the code part below
;****************************************************************************
; Timer setup
;****************************************************************************
; #ifdef QuartzClock
;		ldi	temp,239							; resulting actual frequency division is 239+1=240
; #else
; the value 239 needs to be replaced by different values. The value for the 12s duration then remains 699 so that
; Digit12sFirst (=6), Digit12sMiddle (=9) and Digit12sLast (=9) keep their values.
; ExtendISR12s, ExtendISR1min and ExtendISR5min need to be adjusted via code modification or via configuration.
; Values for some quartzes:
;                  3,500	 3,510	 3,530	 3,540	 3,550	 3,555	 3,560	 3,570	3,579545		 3,600
; Digit12sFirst:		6			6			6			6			6			6			6			6			6				6
; Digit12sMiddle:		9			9			9			9			9			9			9			9			9				9
; Digit12sLast:		9			9			9			9			9			9			9			9			9				9
;
; ExtendISR12s:		3			1			3			1			0			1			3			2			0				2
; ExtendISR1min:		2			4			3			4			1			4			3			0			1				0
; ExtendISR5min:		4			3			0			4			3			5			2			0			0				2
;
; ldi	temp,         233      234      235      236      237      237      237      238      239         240
;

; The 3 digits for the pause between chars in normal operation
; Range between 0 and 199					; !DEP_6!
.EQU DigitPauseFirst = 0
.EQU DigitPauseMiddle = 0
.EQU DigitPauseLast = 0


; The lower 2 digits of the VBat level for VBat value 1 (= battery)
; The upper digit is not configurable since always '1'
.EQU DigitVBat1Middle = 3
.EQU DigitVBat1Last = 0
; => Level = 1,30V


; The lower 2 digits of the VBat level for VBat value 2 (= rechargeable battery)
; The upper digit is not configurable since always '1'
.EQU DigitVBat2Middle = 2
.EQU DigitVBat2Last = 3
; => Level = 1,23V


; Timeout for entering configuration (in ms, range from 0 to 4335ms)
; AVRASM2 doesn't (yet) support floating point arithmetics
.EQU ConfTimeOut = 3500
#define MSTimes(X) ((X)/17)


; how often to repeat a selection until choosing it
#ifndef TEST
.EQU RepeatSelection = 3
#else
.EQU RepeatSelection = 3					; TEST
#endif


; keying speed during configuration (independent of configured keying speed)
.EQU ConfigSpeedDot = 8						; value in dot times => Speed = 5

; keying speed during configuration in 2nd stage (independent of configured keying speed)
.EQU ConfigStage2SpeedDot = 7				; value in dot times => Speed = 6

; max. length of individual configurable callsign (without terminating 255)
; consider the EEPROM size of the various ATtiny devices
; as well as the amount (currently 30) of other data in EEPROM and the required terminating 255
.EQU MaxCallLength = 80

; Start of Key patterns
.EQU SpecChars = (SpecCharsStart-PatternStart)*2
.EQU StringChars = (StringCharsStart-PatternStart)*2
.EQU CharsEnd = (StringCharsEnd-PatternStart)*2
#define KeyVal(X) (((X)-PatternStart)*2)

;****************************************************************************
; Register and Bit Definitions
;****************************************************************************
;***** Working Temporary Register Variables
.DEF temp = r16

;***** Temporary Register Variables
.DEF EEAdr = r17								; used for EEPROM functions
.DEF EEData = r18								; used for EEPROM functions
.DEF BitCounter = r17						; used for CharOut function
.DEF temp2 = r18
.DEF ADCLow = r17
.DEF ADCHigh = r18
.DEF VBatChar1 = r6
.DEF VBatChar2 = r7
.DEF DelayLo = r6								; delay low byte, granularity 30min, range: 0-9,5h
.DEF DelayHi = r7								; delay high byte, granularity 10h, range: 0-50h
.DEF VBatChar3 = r21
.DEF DelayOutInterval = r21				; counting to decide to put out the delay
.DEF temp4 = r21								; used during config.

.DEF ConfigIndex = r15						; which item is being configured
.DEF Delay30 = r15							; delay abs. time, granularity 30min, range: 0-59,5h

;***** ISR Variables
.DEF IRQ_statusreg = r10
.DEF IRQ_LWT_Hi = r19
.DEF IRQ_LWT_Lo = r20
.DEF IRQ_CI = r13

;***** Permanent Register Variables
.DEF SpecCharBits = r1						; the number of bits for a spec. char
.DEF Zero = r2									; constant value zero
.DEF StringLen = r3							; number of chars in the string
.DEF Key = r4									; the programmed Key
.DEF StringPos = r5							; position in string
.DEF One = r8									; constant value one
.DEF Time30 = r9								; operating time since start, granularity 30min

.DEF DotDuration = r11						; duration of a dot in multiple of 0.017152s
.DEF DashDuration = r12						; duration of a dash in multiple of 0.017152s
													; (= 3x duration of a dot)

.DEF StringOutKey = r28						; the returned OutKey when in string mode
.DEF OutKey = r23								; the key to be output, is set to begin 
													; of next char by CharOut function
.DEF WaitTime = r24							; duration low byte to wait by interrupt routine
													; (decremented in interrupt routine)
.DEF SwitchState = r25						; used to check for released switch (debouncing)
.DEF SwitchDuration = r14					; duration of switch activation (could be moved to SRAM)

.DEF Control = r26
;***** Bit Definitions for Control Register
.EQU B_LED 			= 0						; only drive LED
.EQU B_TimerRun 	= 1						; set to enable timer, reset by int. routine 
													; when timer elapsed or switch pressed
.EQU B_TXOverride = 2						; TX is overridden by jumper
.EQU B_CheckSwitch= 3						; Check for Config switch
.EQU B_TXToSet		= 4						; Drive Transmitter and/or LED
.EQU B_Switch 		= 5						; Config switch pressed (always reset in Wait function)
.EQU B_String 		= 6						; set when processing a string

;***** Bit Definitions for IO Registes
#ifdef QuartzClock
.EQU B_IOTX 		= PINB1					; if clocked by the 3,579545MHz clock out of the 74HC00 osc. gate
#else
.EQU B_IOTX 		= PINB3					; for RC oscillator version
#endif
.EQU B_IOLED 		= PINB4
.EQU B_IOSWITCH	= PINB0

.DEF RepChar = r27							; used for key repetitions
.DEF temp3 = r27								; used temporarily within VBat measurement and callsign config.
.DEF VBat = r27								; used temporarily within VBat measurement

.DEF Control2 = r22
;***** Bit Definitions for Control2 Register
.EQU B_DelTimRun = 0							; delay mode active
.EQU B_30TestCycle = 1						; output of delay time

.EQU B_Check12s = 2							; whether to check in Timer ISR for 12s interval switch
.EQU B_Check1min = 3							; whether to check in Timer ISR for 1min interval switch
.EQU B_IntValSw = 4							; indicate that interval switch occured
.EQU B_StopCallsign = 5						; indication to stop callsign transmission
.EQU B_TxIndication = 6						; Config. output on TX and LED

.DEF AddWait = r29							; additional time to wait in CharOut function

;Unused registers:
; none

;****************************************************************************
; RAM layout
;****************************************************************************

.EQU NormalSpeed						= (SRAM_START+0)	; normal speed value
.EQU CallsignSpeed					= (SRAM_START+1)	; speed value for callsign transmission
.EQU Count12s							= (SRAM_START+2)	; counting 12s intervals (0-4)
.EQU Count1min							= (SRAM_START+3)	; counting 1min intervals (0-4)
.EQU Count5min							= (SRAM_START+4)	; counting 5min intervals (0-5)
.EQU CSInterval						= (SRAM_START+5)	; interval for callsign transmission in 12s steps
.EQU Delay12sHi						= (SRAM_START+6)	; 12s duration High byte
.EQU Delay12sLo						= (SRAM_START+7)	; 12s duration Low byte
.EQU VBatLevel							= (SRAM_START+8)	; VBat level for output of either ok or warning
.EQU CharPause							= (SRAM_START+9)	; Pause between output of chars in normal operation
.EQU GeneralConfigStruct			= (SRAM_START+10)	; 4 bytes used during configuration
.EQU EndOf_GeneralConfigStruct	= GeneralConfigStruct + 4
.EQU ConfigDataRAM					= EndOf_GeneralConfigStruct

;***** EEPROM layout and definitions *********************************************************
.EQU	EE_OscMagic = 0						; OscCalMagic numer at address 0, currently not used
.EQU	EE_OscCal	= 1						; oscillator calibration value at address 1, currently not used

.EQU	EE_Version	= 2						; program version at address 2
.EQU	EE_Layout	= 3						; EEPROM layout version at address 3
.EQU	EE_Key		= 4						; key at address 4
.EQU	EE_Speed		= 5						; key speed at address 5 (duration of a dot)
.EQU	EE_VBat		= 6						; VBat configuration at address 6
.EQU	EE_CallsignSpeed = 7					; callsign speed at address 7 (duration of a dot)
.EQU	EE_CallsignMode = 8					; callsign transmission mode at address 8
.EQU	EE_DelayLo	= 9						; start delay low at address 9
.EQU	EE_DelayHi	= 10						; start delay high at address 10
.EQU	EE_DelayMode= 11						; delay mode at address 11
.EQU	EE_TransmissionCycle = 12			; transmission cycle at address 12
.EQU	EE_TransCyclePos = 13				; position within transmission cycle
.EQU  EE_TxIndication = 14					; configuration output by TX or only LED
.EQU  EE_Digit12Last = 15					; last digit of the 12s duration
.EQU  EE_Digit12Middle = 16				; middle digit of the 12s duration
.EQU  EE_Digit12First = 17					; first digit of the 12s duration
.EQU	EE_ExtendISR12s = 18					; ISR cycle duration extension after 12s
.EQU	EE_ExtendISR1min = 19				; ISR cycle duration extension after 1min
.EQU	EE_ExtendISR5min = 20				; ISR cycle duration extension after 5min
.EQU	EE_DigitPauseLast = 21				; last digit of pause between chars
.EQU	EE_DigitPauseMiddle = 22			; middle digit of pause between chars
.EQU	EE_DigitPauseFirst = 23				; first digit of pause between chars
.EQU	EE_DigitVBat1Last = 24				; last digit of VBat1
.EQU	EE_DigitVBat1Middle = 25			; middle digit of VBat1
.EQU	EE_DigitVBat1First = 26				; first digit of VBat1
.EQU	EE_DigitVBat2Last = 27				; last digit of VBat2
.EQU	EE_DigitVBat2Middle = 28			; middle digit of VBat2
.EQU	EE_DigitVBat2First = 29				; first digit of VBat2

.EQU  EE_Call		= 30						; start of individual callsign
;Length of callsign is MaxCallLength plus terminating '255' byte

#define EEDataInRAM(X) ((X)+ConfigDataRAM)
#define ConfigStructData(X) ((X)+GeneralConfigStruct)

;****************************************************************************
;*
;* Main Program
;*
;****************************************************************************
;****************************************************************************
; Vector section
;****************************************************************************
		rjmp	RESET								; 1 = Reset handler
		reti										; 2 = INT0
		reti										; 3 = PCINT0
		reti										; 4 = TIMER1_COMPA
		reti										; 5 = TIMER1_OVF
		reti										; 6 = TIMER0_OVF
		reti										; 7 = EE_RDY
		reti										; 8 = ANA_COMP
		reti										; 9 = ADC
		reti										; 10 = TIMER1_COMPB
		rjmp	TIM0_COMPA						; 11 = TIMER0_COMPA, Timer0 CompareA Handler
		reti										; 12 = TIMER0_COMPB
		reti										; 13 = WDT
		reti										; 14 = USI_START
		reti										; 15 = USI_OVF
;****************************************************************************
; Init
;****************************************************************************
;some inits
;the stack pointer is initialized although its power-on value is ok, but
;it's unclear what happens after a reset caused by a watchdog or brown-out reset
RESET:
		ldi	temp,LOW(RAMEND)
		out	SPL,temp							; set stack pointer (Low) to end of RAM
#ifdef __ATtiny45__
		ldi	temp,HIGH(RAMEND)
		out	SPH,temp							; set stack pointer (High) to end of RAM
#endif

		clr	Control							; init. Control register
		clr	Control2							; init. Control2 register
		clr	Zero								; init. Zero register to zero
		ldi	temp,0x01
		mov	One,temp							; init. One register to one
		out	PORTB,Zero						; is already zero by default
		out	DDRB,Zero						; is already zero by default
		sbi	ACSR,ACD							; switch off Analog Comparator
		sts	Count12s,Zero
		sts	Count1min,Zero
		sts	Count5min,Zero
		clr	IRQ_CI
		clr	Time30

;currently no oscillator calibration
#if 0
;check whether the device runs for the first time after being programmed
		clr 	EEAdr								; OscCalMagic at address 0
		rcall	EERead
		cpi	EEData,OscCalMagic
		breq	EEConf2
;write default OscCalValue
		clr 	EEAdr								; OscCalMagic at address 0
		ldi	EEData,OscCalMagic
		rcall	EEWrite
		in		EEData,OSCCAL
		rcall	EEWrite							; write current Oscillator Calibration Value to address 1
#endif

EEConf2:
		ldi	EEAdr,2							; Version at address 2
		rcall	EERead
		cpi	EEData,Version
		breq	Startup           			; normal Startup
;else program EEPROM with default values
		ldi	ZH,HIGH(EEPROMDefaultValues*2) ; * 2, since flash is adressed bytewise
		ldi	ZL,LOW(EEPROMDefaultValues*2)
		ldi	EEAdr,2							; Version at address 2
l0:	lpm	EEData,Z+
		rcall	EEWrite
#ifndef TEST
		cpi	EEAdr,(EE_Call+1)				; Note: + 1 to include terminating 255 (and postincrement by EEWrite)
#else
		cpi	EEAdr,(EE_Call+1+8)			; TEST: callsign already set, !DEP_7
#endif
		brne	l0
		
;****************************************************************************
; Startup
;****************************************************************************
Startup:
		sbi	PORTB,B_IOSWITCH				; enable pull-up at switch pin
		clr 	EEAdr
		ldi	YH,HIGH(ConfigDataRAM)
		ldi	YL,LOW(ConfigDataRAM)
l51:	rcall	EERead
		st		Y+,EEData
		cpi	EEAdr,EE_Call					; stop at beginning of callsign in EEPROM
		brne	l51

		lds	Key,EEDataInRAM(EE_Key)

		lds	temp,EEDataInRAM(EE_Speed)
		ldi	temp2,13
		sub	temp2,temp
		sts	NormalSpeed,temp2
		
		lds	temp,EEDataInRAM(EE_CallsignSpeed)
		ldi	temp2,13
		sub	temp2,temp
		sts	CallsignSpeed,temp2
		
		rcall	SetDefaultSpeed

		lds	temp,EEDataInRAM(EE_CallsignMode)	; 2,3,4,5: transmit callsign every 5,10,15,20min
		ldi	temp2,231						; !DEP_1! = (0-25) because the dec will
													; be run one time too often
l56:	subi	temp2,(256-25)					; = 'addi' temp2,25
		dec	temp
		brne	l56
		sts	CSInterval,temp2
		
		lds	temp,EEDataInRAM(EE_TxIndication)
		cpse	temp,Zero
		sbr	Control2,1<<B_TxIndication	; Config. output on Tx

		ldi	YH,HIGH(EEDataInRAM(EE_Digit12Last))
		ldi	YL,LOW(EEDataInRAM(EE_Digit12Last))
		rcall	BuildNumber
		sts	Delay12sHi,ZH
		sts	Delay12sLo,ZL
		mov	IRQ_LWT_Hi,ZH					; !DEP_3: IRQ_LWT_xx used afterwards in TIM0_COMPA ISR!
		mov	IRQ_LWT_Lo,ZL

		ldi	YH,HIGH(EEDataInRAM(EE_DigitPauseLast))
		ldi	YL,LOW(EEDataInRAM(EE_DigitPauseLast))
		rcall	BuildNumber
		sts	CharPause,ZL

		ldi	YH,HIGH(EEDataInRAM(EE_DigitVBat1Last))
		ldi	YL,LOW(EEDataInRAM(EE_DigitVBat1Last))
		lds	temp,EEDataInRAM(EE_VBat)
		sbrc	temp,0							; 2 = VBat1, 3= VBat2, don't care for 0 or 1!
		adiw	YH:YL,3							; EE_DigitVBat2Last = EE_DigitVBat1Last + 3
		rcall	BuildNumber
		sts	VBatLevel,ZL
		
		lds	temp,EEDataInRAM(EE_DelayMode)
		tst	temp
		breq	w51								; delay mode: use no delay
		lds	DelayLo,EEDataInRAM(EE_DelayLo)
		lds	DelayHi,EEDataInRAM(EE_DelayHi)
		mov	temp,DelayLo
		or		temp,DelayHi
		breq	w51								; no delay value set
		mov	Delay30,DelayLo
		mov	temp2,DelayHi
		sbr	Control2,1<<B_DelTimRun		; set delay mode
		tst	DelayHi							; Note: optimization with dec and brmi possible
		breq	w51
		ldi	temp,20
l52:	add	Delay30,temp
		dec	temp2								; DelayHi
		brne	l52

;currently no oscillator calibration
#if 0
; Apply oscillator calibration value
w51:	lds	temp2,EEDataInRAM(EE_OscCal)
		tst	temp2
		breq	w50								; don't accept zero as calibration value, something
													; seems to have gone wrong
		in		temp,OSCCAL
l50:	cp		temp,temp2
		breq	w50								; finished, oscillator trimmed to target value
		brlo	w52
;OSCCAL higher than target value in EEPROM
		subi	temp,2
w52:	inc	temp
		out	OSCCAL,temp
		rjmp	l50
#endif

; check for pressed Prog.switch
w51:	sbis	PINB,B_IOSWITCH				; check value at switch pin
		rjmp	CONFIGURE
; switch is not pressed, start normal operation
		
		clt										; use T flag to mark if e0 is entered from CONFIGURE
		clr	AddWait							; init Add. Wait Time register
		lds	temp,EEDataInRAM(EE_DelayMode)
		cpi	temp,1							; maybe EE_DelayMode was 1, then leave delay
													; mode if switch was not pressed
		brne	e0
		cbr	Control2,1<<B_DelTimRun		; leave delay mode
		
;****************************************************************************
; Entry Point for normal operation (either with or without delay)
;****************************************************************************
e0:	cbr	Control,1<<B_LED				; disable LED indication only

		lds	temp,EEDataInRAM(EE_TransmissionCycle)
		cpi	temp,3							; wait for trigger pin?
		brlo	e2									; no, start operation
		subi	temp,2
		sts	EEDataInRAM(EE_TransmissionCycle),temp
		brts	e3
e0l:	sbic	PINB,B_IOSWITCH				; check value at switch pin
		rjmp	e0l
	
e2:	brts	e3									; don't enter TimerSetup again because we
													; came from CONFIGURE
		rcall	TimerSetup						; enables also the interrupts

e3:	lds	temp,EEDataInRAM(EE_VBat)
		cpse	temp,Zero						; skip battery voltage measurement if 0
		rcall	BatVoltage						; measure battery voltage
		ldi 	temp,(1<<PRTIM1)|(1<<PRUSI)|(1<<PRADC)	; shut down timer1, USI and ADC modules
		out	PRR,temp
e1:	wdr										; reset watchdog before activating it
		ldi	temp,0b00111001				; WDE = 1, WDCE=1, WDP[3..0] = 1001, timeout 8s
		out	WDTCR,temp						; enable watchdog

		cbi	PORTB,B_IOSWITCH				; disable pull-up at switch pin
		cbr	Control,1<<B_LED				; disable LED indication only
		
		in		temp,MCUSR						; check source of reset
		out	MCUSR,Zero						; reset MCU status register, clear all reset flags
		sbrc	temp,WDRF
		cbr	Control2,1<<B_DelTimRun		; OUCH! a watchdog reset occured, skip any delay and go
													; right away to normal operation
		; enter Normal Operation
		
		
;****************************************************************************
; Normal Operation, with or without delay
;****************************************************************************

s1i:	clr	DelayOutInterval
s1l:	inc	DelayOutInterval
		andi	DelayOutInterval,3			; DelayOutInterval runs from 3 to 0
s1:	sbrs	Control2,B_DelTimRun
		rjmp	s19
		wdr
		sbr	Control2,1<<B_30TestCycle
		cpse	Time30,Zero
		cbr	Control2,1<<B_30TestCycle
		cpse	DelayOutInterval,Zero
		cbr	Control2,1<<B_30TestCycle	; no delay output if DelayOutInterval != 0
		tst	Time30
		breq	s19								; transmit in first 30min
		cp		Delay30,Time30					; Delay passed?
		brne	s1									; not yet, wait
		cbr	Control2,1<<B_DelTimRun		; deactivate delay mode
		
s19:	clt
		lds	temp,EEDataInRAM(EE_TransmissionCycle)
		tst	temp
		breq	s16								; continuous transmission
		lds	temp2,EEDataInRAM(EE_TransCyclePos)
		dec	temp2								; because of storage between 1-5
		cpi	temp,2							; 5min cycle?
		breq	s17
; 12s interval, 1min cycle
s18:	lds	temp,Count12s					; Count12s changed by ISR
		wdr										; trigger watchdog
		cp		temp,temp2
		brne	s18
		sbr	Control2,1<<B_Check12s		; activate transmission cut off
		cpi	IRQ_LWT_Hi,2					; beyond the first 3,2s of the 12s interval?
		brne	s16								; yes
		rjmp	s11								; no
; 1min interval, 5min cycle
s17:	lds	temp,Count1min					; Count1min changed by ISR
		wdr										; trigger watchdog
		cp		temp,temp2
		brne	s17
		sbr	Control2,1<<B_Check1min		; activate transmission cut off
		lds	temp,Count12s
		cpi	temp,4							; are we in the last 12s of the actual 1min transmission interval?
		brne	s11								; no

s16:	set										; use T flag to remember callsign output
s11:	lds	temp,EEDataInRAM(EE_CallsignMode)
		tst	temp
		brne	s12
; char/string output
s14:	rcall	SetNormalSpeed
		sbrc	Control2,B_30TestCycle
		rjmp	deo								; jump into DelayConfig to put out the configured delay
		mov	OutKey,Key
		lds	AddWait,CharPause
		rcall	CharOut
		rjmp	s1l

s12:	cpi	temp,1							; only callsign output?
		breq	s15								; yes

;output of char/string and callsign
s13:	brtc	s14								; callsign transmission not now
		tst	IRQ_CI							; has callsign interval expired?
		brne	s14								; no, normal output
		lds	IRQ_CI,CSInterval				; restart callsign interval
		; and output callsign (with priority over delay output)
; callsign output
s20:	rcall	SetCallsignSpeed
		rcall CallSignOut
		rjmp	s1l		

; callsign output if not output of delay
s15:	rcall	SetNormalSpeed
		sbrc	Control2,B_30TestCycle
		rjmp	deo								; jump into DelayConfig to put out the configured delay
		rjmp	s20								; jump to callsign output


;****************************************************************************
; Build numerical value
;****************************************************************************
; build a one or two byte number from three digits stored in EEPROM/RAM
; pointer to first digit in RAM is passed in Y reg.
; result is in Z reg.
BuildNumber:
		ld		ZL,Y+								; start with last (least sign.) digit
		clr	ZH
		ld		temp,Y+							; middle digit
l100:	dec	temp
		brmi	w100
		adiw	ZH:ZL,10
		rjmp	l100
w100:	ld		temp,Y							; first digit
l101:	dec	temp
		brmi	w101
		adiw	ZH:ZL,50							; adiw is limited to adding max. 63
		adiw	ZH:ZL,50
		rjmp	l101
w101:	ret

;****************************************************************************
; Timer setup
;****************************************************************************
TimerSetup:
		ldi	temp,(1<<CS02)					; CK/256
		out	TCCR0B,temp
		ldi	temp,(1<<WGM01)				; CTC mode
		out	TCCR0A,temp
#ifdef QuartzClock
		ldi	temp,239							; resulting actual frequency division is 239+1=240
#else
		ldi	temp,66							; resulting actual frequency division is 66+1=67
#endif
		out	OCR0A,temp
		ldi	temp,(1<<OCIE0A)				; OCIE0A
		out	TIMSK,temp
		sei										; enable interrupts, i.e. the timer interrupt
		ret

;****************************************************************************
; Configuration
;****************************************************************************
CONFIGURE:
		set										; mark that we were in CONFIGURE
		rcall	TimerSetup						; enables also the interrupts
#ifdef LedOnlyConfig
		sbr	Control,1<<B_LED				; disable TX, only LED indication
#else
		sbrs	Control2,B_TxIndication
		sbr	Control,1<<B_LED				; disable TX, only LED indication
#endif
		ldi	WaitTime,MSTimes(ConfTimeOut)	; wait for max. the config. time
		ldi	SwitchState,0x03				; set switch state initially
		sbr	Control,(1<<B_TimerRun)|(1<<B_TXToSet)	; start waiting and drive LED to show reaction
		rcall SetTX
l40:	sbrs	Control,B_TimerRun			; wait for operator to release switch
		rjmp	e0									; not released within TIMEOUT -> Entry Point for Delay Jumper	TBD
		tst	SwitchState
		brne	l40								; still pressed

		rcall SetTXOff							; turn off LED
		sbr	Control,1<<B_CheckSwitch	; enable switch checking

; output version to indicate the initialization
		ldi	OutKey,CharsEnd				; output version information (string)
		ldi	AddWait,20						; add a pause after the version string
		rcall	CharOut
		; enter general configuration below

;**************************
;General Configuration
;**************************
;use 'Key' since restarting after callsign configuration

		clr	ConfigIndex

;Copy config data structure (4 bytes) to RAM
w78:	ldi	ZH,HIGH(ConfigDataSelection*2)	; * 2, since flash is adressed bytewise
		ldi	ZL,LOW(ConfigDataSelection*2)
		mov	temp,ConfigIndex				; ConfigIndex * 4
		lsl	temp
		lsl	temp
		add	ZL,temp
		adc	ZH,Zero
		ldi	YH,HIGH(GeneralConfigStruct)
		ldi	YL,LOW(GeneralConfigStruct)
		ldi	temp,4
l71:	lpm	R0,Z+
		st		Y+,R0
		dec	temp
		brne	l71

		tst	R0									; R0 contains configoffset
		breq	w79								; start with start value
		
		ldi	YH,HIGH(ConfigDataRAM)
		ldi	YL,LOW(ConfigDataRAM)
		add	YL,R0								; R0 still valid from above
		adc	YH,Zero
		ld		Key,Y								; start with stored value

w71:	ldi	RepChar,RepeatSelection		; how often to repeat a selection until choosing it
l72:	mov	OutKey,Key
		cp		ConfigIndex,One				; key configuration?
		breq	w75
		
		cpi	OutKey,10						; < 10?
		brlo	w72								; then only put out one digit
		mov	temp4,Key
		ldi	OutKey,DK('0')					; start with '0'
l73:	inc	OutKey							; and increment right away by one
		subi	temp4,10
		cpi	temp4,10							; < 10?
		brsh	l73
		rcall	CharOut							; output first char
		sbrc	Control,B_Switch
		rjmp	w73								; Switch pressed => proceed to next key
		mov	OutKey,temp4
w72:	subi	OutKey,(256-26)				; add 26
w75:	ldi	AddWait,60						; pause after each sequence of chars
		cpse	RepChar,One						; but only after the last sequence
		ldi	AddWait,30						; else add a smaller pause after each number
		rcall	CharOut
		sbrc	Control,B_Switch
		rjmp	w73								; Switch pressed => proceed to next key
		dec	RepChar
		brne	l72
		rjmp	w70								; Switch not pressed in time => store key
w73:	cp		ConfigIndex,One				; key configuration?
		brne	w74
		sbrc	SwitchDuration,7				; Switch pressed for more than 2.1s?
		ldi	OutKey,MOEKey					; yes, jump to MOE as next key
		mov	Key,OutKey
		rjmp	w71
w74:	lds	temp,ConfigStructData(1)	; endvalue
		cp		temp,Key
		brne	w76
w79:	lds	Key,ConfigStructData(0)		; startvalue
		rjmp	w71
w76:	inc	Key
		rjmp	w71
;the section below can't be moved up above w73 in order to get rid of the 'rjmp w70' because
;the relative branch to DelayConfig below is then out of reach
w70:	tst	ConfigIndex						; at first config. stage?
		brne	w77								; no
		rcall	SetConfigStage2Speed			; yes, change keying speed to signal being in 2nd config. stage
		mov	temp,Key
		cpi	temp,4							; callsign configuration?		!DEP_4!
		breq	CallsignConfig
		cpi	temp,7							; delay configuration?			!DEP_5!
		breq	DelayConfig
w710:	mov	ConfigIndex,Key
		rjmp	w78

; store the selected config. value
w77:	lds 	EEAdr,ConfigStructData(3)	; configoffset
		mov	EEData,Key
		rcall	EEWrite
		
		lds	Key,ConfigStructData(2)		; continue to another configuration?
		tst	Key
		brne	w710								; yes, since Key was used go to w710

ConfExit:
		cbr	Control,1<<B_CheckSwitch	; disable switch checking
		ldi	WaitTime,MSTimes(2500)		; add another longer pause (2,5s) to indicate programming
		rcall	Wait
		cli
		rjmp	RESET								; restart Tinyfox => read in the modified configuration

;**********************
;Callsign Configuration
;**********************
;use temp4 for the key
CallsignConfig:
		ldi	EEAdr,EE_Call
l80:	ldi	temp,KeyVal(SDahString)
w85:	mov	temp4,temp

		ldi	RepChar,RepeatSelection		; how often to repeat a char until choosing it
l82:	mov	OutKey,temp4
		cpi	RepChar,1
		brne	w81
		ldi	AddWait,60						; add another pause after each sequence of chars
w81:	push	EEAdr								; r17=EEAdr used in CharOut
		rcall	CharOut
		pop	EEAdr
		sbrc	Control,B_Switch
		rjmp	w82								; Switch pressed => proceed to next char
		dec	RepChar
		brne	l82

; Switch not pressed in time and no more char repetitions => store selected char
		cpi	temp4,KeyVal(SDahString)
		breq	w87
		ldi	EEData,254
		cpi	temp4,KeyVal(PDahString)
		breq	w86
		mov	EEData,temp4
w86:	rcall	EEWrite							; store selected char
w87:	ldi	EEData,255
		rcall	EEWrite							; always append the terminating char
		dec	EEAdr								; but overwrite it with the next char
		cpi	EEAdr,(EE_Call+MaxCallLength)
		breq	ConfExit							; max. length reached, exit callsign config.
		cpi	temp4,KeyVal(SDahString)	; configuration finished by user?
		breq	ConfExit							; yes
		rjmp	l80								; else configure another char

; Switch pressed => proceed to next char
w82:	ldi	temp,DK('0')
		sbrc	SwitchDuration,7				; Switch pressed for more than 2.1s?
		rjmp	w85								; yes, jump to '0'
		ldi	temp,KeyVal(PDahString)
		cpi	temp4,KeyVal(SDahString)
		breq	w85
		clr	temp								; 'a'
		cpi	temp4,KeyVal(PDahString)
		breq	w85
		tst	OutKey							; after the last selectable string, switchover to 'a' = 0
		breq	l80
		mov	temp,OutKey
		rjmp	w85

;*******************
;Delay Configuration
;*******************
DelayConfig:
		clr	DelayLo
wl47:	clr	DelayHi
; deo is the entry point from Normal Operation in Delay Mode
deo:	ldi	OutKey,(Correction-PatternStart)*2	; correction tone (8 dots), to indicate subsequent
																; output of delay time
		sbrc	Control2,B_30TestCycle		; ignore when not in delay mode
		rcall	CharOut
		sbrc	Control2,B_StopCallsign		; stop transmission, can only happen in normal operation
		rjmp	s1l
l47:	ldi	RepChar,RepeatSelection		; how often to repeat a delay time until storing the delay value
; put out current delay
l46:	tst	DelayHi
		breq	w45
		mov	OutKey,DelayHi
		rcall	DigitOut
		sbrc	Control,B_Switch
		rjmp	IncDelay							; Switch pressed => increment Delay
		sbrc	Control2,B_StopCallsign		; stop transmission, can only happen in normal operation
		rjmp	s1l
w45:	ldi	OutKey,26<<1					; 2 * offset for '0' because of lsr below
		add	OutKey,DelayLo
		lsr	OutKey							; because the granularity is 30min
		sbrs	DelayLo,0						; check for 30min, could also be done by checking the carry flag
		rjmp	w46
		rcall	CharOut
		sbrc	Control2,B_StopCallsign		; stop transmission, can only happen in normal operation
		rjmp	s1l
		sbrc	Control,B_Switch
		rjmp	IncDelay							; Switch pressed => increment Delay
		ldi	OutKey,(Daaah-PatternStart)*2	; long tone, special 30min indicator
w46:	ldi	AddWait,60						; add another pause after each sequence of chars
		cpse	RepChar,One
		mov	AddWait,DashDuration			; add another pause (3 dots long) after the complete number
		rcall	CharOut
		sbrc	Control2,B_30TestCycle
		rjmp	s1l								; jump back to Normal Operation
		sbrc	Control,B_Switch
		rjmp	IncDelay							; Switch pressed => increment Delay
		
		dec	RepChar
		brne	l46

; store actual delay in EEPROM
		ldi 	EEAdr,EE_DelayLo
		mov	EEData,DelayLo
		rcall	EEWrite							; EEAdr is incremented by EEWrite
		mov	EEData,DelayHi
		rcall	EEWrite
		rjmp	ConfExit							; go to Conf. Exit above
		
IncDelay:
		mov	temp,DelayLo
		inc	DelayLo
		or		temp,DelayHi					; are DelayLo and DelayHi both 0?
		breq	IncDelay							; yes, skip the 30min and go from the 0h to 1h directly
		ldi	temp,20
		cp		DelayLo,temp					; DelayLo only between 0 and 19 (=9,5h)
		brne	l47
		clr	DelayLo
		inc	DelayHi
		ldi	temp,6
		cp		DelayHi,temp					; DelayHi only between 0 and 5 (=50h)
		brne	l47
		rjmp	wl47

;****************************************************************************
; Timer interrupt service routine
;****************************************************************************
; This function is called after 67 (Timer Compare Match) * 256 (Clock Prescaler)
; oscillator ticks for the RC oscillator
; The values are based on the assumption of a calibrated ATtiny25/45 at 3.3V that runs
; at 1MHz (internal RC oscillator with nominal 8MHz and a 8 divider)
; In QUARTZ mode this function is called after 240 (Timer Compare Match) * 256 (Clock Prescaler)
; oscillator ticks
; That results in an invocation of this function of 0.017152s
; Short delays are therefore selectable between 0.017152s and 4.373s
TIM0_COMPA:
		in		IRQ_statusreg,SREG			; save Status register
		push	temp								; save temp on stack
		push	temp2								; save temp2 on stack
		tst	SwitchState
		breq	w22
		sbic	PINB,B_IOSWITCH				; check switch
		rjmp	w27								; not pressed
		ldi	SwitchState,0x03				; still pressed -> set switch state (+1 for dec below)
		sbrs	SwitchDuration,7				; increment duration of switch activation only if <128
		inc	SwitchDuration					; increment duration of switch activation
w27:	dec	SwitchState		
w22:	sbrs	Control,B_TimerRun
		rjmp	w20								; no running short delay timer
		dec	WaitTime
		brne	w20
		cbr	Control,1<<B_TimerRun

w20:	sub	IRQ_LWT_Lo,One					; !DEP_3: initially set in startup code!
		sbc	IRQ_LWT_Hi,Zero
		tst	IRQ_LWT_Hi
		brne	w24
		tst	IRQ_LWT_Lo
		brne	w24
;12s interval has passed
		cpse	IRQ_CI,Zero
		dec	IRQ_CI

		lds	IRQ_LWT_Hi,Delay12sHi
		lds	IRQ_LWT_Lo,Delay12sLo
		lds	temp,Count12s
		lds	temp2,EEDataInRAM(EE_ExtendISR12s)
; TBD OPT: the code below can be optimized by moving into a subroutine, as the same piece of code occurs three times
		cp		temp,temp2
		brsh	w23								; allow only 4 increments, because 5 would be equal
													; to setting Delay12sLo one higher
		add	IRQ_LWT_Lo,One					; add one
		adc	IRQ_LWT_Hi,Zero				; don't rely on values that allow to use 'inc'
w23:	inc	temp
; TBD OPT: end of optimization
		sts	Count12s,temp
		sbrc	Control2,B_Check12s			; shall 12s switch be indicated?
		sbr	Control2,1<<B_IntValSw		; yes
		cpi	temp,5
		brne	w24
		sts	Count12s,Zero

		lds	temp,Count1min
		lds	temp2,EEDataInRAM(EE_ExtendISR1min)
		cp		temp,temp2
		brsh	w25								; allow only 4 increments, because 5 would be equal
													; to setting ExtendISR12s one higher
		add	IRQ_LWT_Lo,One					; add one
		adc	IRQ_LWT_Hi,Zero				; don't rely on values that allow to use 'inc'
w25:	inc	temp
		sts	Count1min,temp
		sbrc	Control2,B_Check1min			; shall 1min switch be indicated?
		sbr	Control2,1<<B_IntValSw		; yes
		cpi	temp,5
		brne	w24
		sts	Count1min,Zero

		lds	temp,Count5min
		lds	temp2,EEDataInRAM(EE_ExtendISR5min)
		cp		temp,temp2
		brsh	w26								; allow only 5 increments, because 6 would be equal
													; to setting ExtendISR1min one higher
		add	IRQ_LWT_Lo,One					; add one
		adc	IRQ_LWT_Hi,Zero				; don't rely on values that allow to use 'inc'
w26:	inc	temp
		sts	Count5min,temp
		cpi	temp,6							; count until 6 (=30min)
		brne	w24
		sts	Count5min,Zero
		
		inc	Time30							; increment passed time by 30min

w24:	pop	temp2								; restore temp2 from stack
		pop	temp								; restore temp from stack
		out	SREG,IRQ_statusreg			; restore Status register
		reti

;****************************************************************************
; EEPROM service routines
;****************************************************************************
EERead:
		sbic 	EECR,EEPE						; if EEPE not clear
		rjmp	EERead							; wait more
		out 	EEARL,EEAdr						; set EEPROM address
		out	EEARH,Zero						; clear upper address byte
		sbi	EECR,EERE						; set EEPROM Read strobe
		in		EEData,EEDR						; get data
		rjmp	eeo

EEWrite:
		in		temp,SREG						; save Status register
		cli										; disable interrupts before writing to EEPROM
el1:	sbic	EECR,EEPE						; wait for EEPE to be cleared
		rjmp	el1								; just for safety, since we wait also at the end
		out	EECR,Zero						; setting EEPM0 and EEPM1 both to 0, this maybe unnecessary
		out 	EEARL,EEAdr						; set EEPROM address
		out	EEARH,Zero						; clear upper address byte
		out 	EEDR,EEData						; set EEPROM data
		sbi 	EECR,EEMPE						; set master program enable
		sbi	EECR,EEPE						; set EEPROM Program strobe
el2:	sbic	EECR,EEPE						; wait for EEPE to be cleared
		rjmp	el2
		out	SREG,temp						; restore Status register, that will enable interrupts again
													; if they were enabled before
eeo:	inc	EEAdr								; increment address for subsequent write (or read) operations
		ret

;****************************************************************************
; Callsign output
;****************************************************************************
CallsignOut:
		wdr										; always reset watchdog (for special case:
													; no chars configured at all)
		ldi	EEAdr,EE_Call
l90:	rcall	EERead
		cpi	EEData,254						; pause?
		brne	w90
		mov	WaitTime,DashDuration		; pause duration, depends on keying speed
		add	WaitTime,DotDuration			; in total 4 more dot durations, since distance between words is 7
		rcall	Wait
		rjmp	l90
w90:	cpi	EEData,255						; termination?
		breq	re2
		mov	OutKey,EEData
		push	EEAdr								; r17=EEAdr used in CharOut
		lds	temp,EEDataInRAM(EE_CallsignMode)
		cpi	temp,1							; only callsign output?
		brne	w91								; no
		lds	AddWait,CharPause				; yes, then add configured pause
w91:	rcall	CharOut
		pop	EEAdr
		sbrs	Control2,B_StopCallsign
		rjmp	l90
re2:	ret

;****************************************************************************
; Character output routines
;****************************************************************************
ProcString:
		sbrc	Control,B_String				; are we already in string mode ?
		rjmp	nchar								; ignore
		sbr	Control,1<<B_String			; set string mode
		mov	StringLen,R0
		mov	StringOutKey,OutKey
		add	StringOutKey,StringLen
		cpi	StringOutKey,CharsEnd
		brlo	w7
		clr	StringOutKey					; set to first normal char ('a')
w7:	mov	StringPos,OutKey				; keep track of actual char

w8:	ldi	ZH,HIGH(PatternStart*2)
		ldi	ZL,LOW(PatternStart*2)
		add	ZL,StringPos
		adc	ZH,Zero
		lpm	OutKey,Z
		rjmp	CharOutInt

;if key was not pressed
cout:
		sbrs	Control,B_String				; are we in string mode
		rjmp	w9									; no, return
nchar:
		inc	StringPos
		dec	StringLen
		brne	w8
; finished string
		mov	OutKey,StringOutKey
		add	AddWait,DashDuration			; additional pause after a word (3 dots long)	!DEP_6!
; check if additional wait time is set
w9:	tst	AddWait
		breq	o1									; if zero -> exit function
		mov	WaitTime,AddWait
		clr	AddWait
		rjmp	Wait								; don't check switch, return anyway
													; no need to check B_IntValSw flag since no transmission here
		; !OPT: rcall Wait was changed to rjmp Wait!

;if key was pressed
;no need to clear AddWait because AddWait too small in Config mode
kout:
		sbrc	Control,B_String				; are we in string mode
		mov	OutKey,StringOutKey			; yes, update OutKey
o1:	ret

;if switch to next transmission interval occured
iout:	rcall SetTXOff							; turn off TX
sout:	cbr	Control2,1<<B_IntValSw		; remove trigger
		sbr	Control2,1<<B_StopCallsign	; stop if callsign is being transmitted
		clr	AddWait							; clear any set add. wait time
		ret

; entry point for digits: add 26 (='0') to the value
DigitOut:
		subi	OutKey,(256-26)				; equal to adding 26
; normal entry point
CharOut:
		cbr	Control,1<<B_String			; clear string mode
		cbr	Control2,1<<B_StopCallsign	; clear callsign transmission stop
		cpse	IRQ_LWT_Hi,Zero				; within the first 8s of every 12s interval:
		cbr	Control2,1<<B_IntValSw		; remove any pending trigger
; entry point from string processing
CharOutInt:
		ldi	ZH,HIGH(PatternStart*2) 	; * 2, since flash is adressed bytewise
		ldi	ZL,LOW(PatternStart*2)
		add	ZL,OutKey
		adc	ZH,Zero
		lpm	R0,Z+								; optimization for access in ProcSpecChars below	!DEP_2!
		mov	temp,OutKey
		inc	OutKey
		tst	R0
		brne	w6
		cpi	OutKey,CharsEnd
		brlo	CharOutInt						; proceed to next memory location
		clr	OutKey
		rjmp	CharOutInt						; proceed to next memory location
w6:	cpi	temp,StringChars
		brsh	ProcString
		cpi	temp,SpecChars
		brsh	ProcSpecChars
		ldi	BitCounter,0x08
l2:	dec	BitCounter
		breq	CharOutInt						; empty char -> proceed to next one
													; don't put empty chars inside strings !
		lsl	R0
		brcc	l2
w1:	sbr	Control,1<<B_TXToSet			; turn on TX
		rcall SetTX
		mov	WaitTime,DashDuration		; a dash (= 3 dots long)
		lsl	R0
		brcs	w2
		mov 	WaitTime,DotDuration			; a dot
w2:	rcall Wait
		sbrc	Control2,B_IntValSw
		rjmp	iout								; switch to next trans. interval -> exit function
		sbrc	Control,B_Switch
		rjmp	kout								; config switch pressed -> exit function
		rcall SetTXOff							; turn off TX
		mov	WaitTime,DotDuration			; wait one dot long between the dots and dashes of a char
		dec	BitCounter
		brne	w3
		mov	WaitTime,DashDuration		; wait 3 dots long after a char
w3:	rcall Wait
		sbrc	Control2,B_IntValSw
		rjmp	sout								; switch to next trans. interval -> exit function
		sbrc	Control,B_Switch
		rjmp	kout								; config switch pressed -> exit function
		tst	BitCounter
		brne	w1
		rjmp	cout								; all bits processed -> exit normal bit processing

ProcSpecChars:
		clr 	BitCounter
		mov	SpecCharBits,R0
		dec	R0
		lsr	R0
		lsr	R0
		lsr	R0
		add	OutKey,R0
		inc	OutKey
l3:	ldi	temp,0x07
		and	temp,BitCounter
		brne	w4
		lpm	R0,Z+								; Z was post-incremented above	!DEP_2!
w4:	cbr	Control,1<<B_TXToSet
		lsl	R0
		brcc	w5
		sbr	Control,1<<B_TXToSet
w5:	rcall SetTX
		mov	WaitTime,DotDuration			; wait one dot long, this is the base unit
		rcall Wait
		sbrc	Control2,B_IntValSw
		rjmp	iout								; switch to next trans. interval -> exit function
		sbrc	Control,B_Switch
		rjmp	kout								; config switch pressed -> exit function
		inc	BitCounter
		cp		BitCounter,SpecCharBits
		brne	l3
		rjmp	cout								; all bits processed -> exit spec. bit processing

;****************************************************************************
; Wait
;****************************************************************************
Wait:
		wdr										; always reset watchdog
		cbr	Control,1<<B_Switch			; always reset switch flag
		sbr	Control,1<<B_TimerRun

l30:	sbrc	Control,B_TimerRun			; time elapsed?
		sbrc	Control2,B_IntValSw
		ret										; time elapsed or abort because of switch to next transmission interval

		sbrc	Control,B_CheckSwitch
		cpse	SwitchState,Zero				; was cleared in interrupt service routine ?
													; the line above is questionable
		rjmp	l30								; don't check switch or switch state was not 
													; yet cleared in int. service routine
		sbic	PINB,B_IOSWITCH				; check switch
		rjmp	l30								; switch is not pressed, keep on waiting
		clr	AddWait							; reset add. wait time
		sbr	Control,(1<<B_Switch)|(1<<B_TXToSet)	; set switch flag and turn on TX
		ldi	SwitchState,0x02				; set switch state
		clr	SwitchDuration
		rcall SetTX
l31:	tst	SwitchState						; check for switch state (will be reset in int. routine)
		brne	l31
		rcall SetTXOff							; turn off TX, no irq locking necessary, 
													; since int. routine was executed just before; TBD: remove
		ldi	WaitTime,30						; add a pause before doing anything else
		sbr	Control,1<<B_TimerRun
l32:	sbrc	Control,B_TimerRun
		rjmp	l32
		ret
		
;****************************************************************************
; Drive LED and TX
;****************************************************************************
; PB0 = config. switch, input with selectable internal pull-up
; if using int. RC osc.:
; 		PB3 = tx,  tristate if disabled, push one for transmitting
; if clocked by the 3,579545MHz clock out of the 74HC00 osc. gate:
; 		PB1 = tx,  tristate if disabled, push one for transmitting
; PB4 = led, tristate if disabled, pull zero for lighting led (=normally when transmitting)
; at LED only Configuration the TX is not keyed, if the TX is keyed then the switch will
; not be checked, therefore no need to activate the pull-up at the switch
; if Configuration with active TX, then the switch pull-up needs to be activated
SetTXOff:										; entry point to turn off the TX
		cbr	Control,1<<B_TXToSet			; turn off TX
SetTX:											; entry point to derive the TX setting from B_TXToSet
		sbrc	Control,B_LED
		rjmp	w11								; only LED
		sbic	DDRB,B_IOTX						; check for TX override by user while TX is input
													; (i.e. not actively driven because not keyed)
		rjmp	w10
		cbr	Control,1<<B_TXOverride
		sbic	PINB,B_IOTX						; is TX externally driven high?
		sbr	Control,1<<B_TXOverride		; yes: overridden by jumper, set B_TXOverride bit
w10:	sbrc	Control,B_TXToSet
		rjmp	TXOn
;TXOff
#ifdef LedOnlyConfig
		ldi	temp,0x00
#else
		ldi	temp,0x00
		sbrc	Control2,B_TxIndication
		ldi	temp,1<<B_IOSWITCH			; 0x01 for config. switch pull-up
#endif
		ldi	temp2,1<<B_IOLED				; 0x10, prepare for LED indication in case of TX override
		sbrc	Control,B_TXOverride
		rjmp	w13
w12:	out	DDRB,Zero						; switch off: first set to inputs
		out	PORTB,temp						; then set pull-ups
		ret
#ifdef LedOnlyConfig		
TXOn:	ldi	temp,1<<B_IOTX
#else
TXOn:	ldi	temp,1<<B_IOTX					; 0x08 (RC) / 0x02 (ext. osc)
		sbrc	Control2,B_TxIndication
		ldi	temp,(1<<B_IOTX)|(1<<B_IOSWITCH)	; 0x09 (RC)/0x03 (ext. osc.) for TX and config. switch pull-up
#endif
		ldi	temp2,(1<<B_IOLED)|(1<<B_IOTX)	; 0x18 (RC) / 0x12 (ext. osc)
w13:	out	PORTB,temp						; switch on: first prepare outputs
w14:	out	DDRB,temp2						; then activate output
		ret

;only LED
w11:	clr	temp2
		sbrc	Control,B_TXToSet
		ldi	temp2,(1<<B_IOLED)			; 0x10
		rjmp	w14

;****************************************************************************
; Set keying speed
;****************************************************************************
SetDefaultSpeed:
		ldi	temp,ConfigSpeedDot			; use default for values (for configuration if required)
		rjmp	wss2
SetConfigStage2Speed:
		ldi	temp,ConfigStage2SpeedDot	; for configuration in 2nd stage
wss2:	mov	DotDuration,temp
		rjmp	wss
SetCallsignSpeed:
		lds	DotDuration,CallsignSpeed
		rjmp	wss
SetNormalSpeed:
		lds	DotDuration,NormalSpeed
wss:	mov	DashDuration,DotDuration
		add	DashDuration,DotDuration
		add	DashDuration,DotDuration	; DashDuration = 3x DotDuration
		ret

;****************************************************************************
; Measure battery voltage
;****************************************************************************
BatVoltage:
		push	r6
		push	r7
		sbi	ADCSRA,ADEN						; enable ADC
		out	ADMUX,One						; 0x01: channel = ADC1 (PB2)
		sbi	ADCSRA,ADPS2					; ADC Prescaler = 16

		sbr	Control,1<<B_TXToSet			; turn on TX
		rcall SetTX								; drive TX to increase current consumption
		ldi	WaitTime,MSTimes(4000)		; for 4s
		sbr	Control,1<<B_TimerRun
l65:	sbrc	Control,B_TimerRun
		rjmp	l65								; still wait
		clr	ADCLow
		clr	ADCHigh
		ldi	temp3,36
l61:	sbi	ADCSRA,ADSC						; start measurements (first one can not be used)
l60:	sbic	ADCSRA,ADSC						; wait for conversion to be finished
		rjmp	l60
		cpi	temp3,33
		brsh	w61								; if greater or equal 33 drop measurements, i.e. first 4 measurements are dropped
		in		temp,ADCL
		add	ADCLow,temp
		in		temp,ADCH
		adc	ADCHigh,temp		
w61:	dec	temp3
		brne	l61

		push	ADCHigh							; ADCHigh = temp2 which is used in SetTXOff funct. below
		rcall SetTXOff							; turn off TX
		ldi	WaitTime,60						; add a pause before doing anything else
		rcall Wait
		pop	ADCHigh

; Conversion:
; Vin = ADC*Vref/1024*100 = ADC*330/1024 = ADC*165/512 = ADC/4+ADC/16+ADC/128+ADC/512 (= (128+32+4+1)/512*ADC = 165/512*ADC)
; Experience shows however that the MAX1724EZK33 has an output of about 3,32V in the Tinyfox TX:
; Vin = ADC*Vref/1024*100 = ADC*332/1024 = ADC*166/512 = ADC/4+ADC/16+ADC/128+ADC/256 (= (128+32+4+2)/512*ADC = 166/512*ADC)
; ADC/4: 2 shifts + 5 shifts because of averaging over 32 measurements
		ldi	temp3,7
l64:	lsr	ADCHigh
		ror	ADCLow
		dec	temp3
		brne	l64
		mov	VBat,ADCLow
; another two shifts: ADC/4 => ADC/16
		lsr	ADCLow							; no more bits in ADCHigh left to shift in
		lsr	ADCLow
; and then add it
		add	VBat,ADCLow
; another three shifts: ADC/16 => ADC/128
		lsr	ADCLow
		lsr	ADCLow
		lsr	ADCLow
; and then add it
		add	VBat,ADCLow
; another shift: ADC/128 => ADC/256
		lsr	ADCLow
; and then add it
		add	VBat,ADCLow

; compare VBat with warning level
		clt										; use 'T' flag to track warning state
		lds	temp,VBatLevel
		cp		VBat,temp
		brsh	w62
		set										; set 'T' flag to indicate VBat<warning level

; convert number in VBat to three digits representing the number
w62:	ldi	VBatChar3,2						; last digit, start with 002 because of breq 
													; after dec below and a correction of plus 1
		ldi	temp2,10
		clr	VBatChar1						; first digit
l67:	clr	VBatChar2						; middle digit
l62:	dec	VBat								; VBat can not be equal to 0!
		breq	w60
		inc	VBatChar3
		cp		VBatChar3,temp2
		brne	l62
		clr	VBatChar3
		inc	VBatChar2
		cp		VBatChar2,temp2
		brne	l62
		inc	VBatChar1
		rjmp	l67

w60:	lds	temp,EEDataInRAM(EE_VBat)	; Show battery warning state?
		ldi	temp3,2							; output VBat two times, preload for below
		cpi	temp,1
		breq	l63								; no
		ldi	temp3,4							; 4 repetitions
l66:	ldi	OutKey,(Daaah-PatternStart)*2		; long tone
		brtc	w64								; no warning
		ldi	OutKey,(Correction-PatternStart)*2	; correction tone (8 dots)
w64:	cpi	temp3,1
		brne	w65
		ldi	AddWait,50						; add pause after warning state output
w65:	rcall	CharOut
		dec	temp3
		brne	l66
		ldi	temp3,1							; output VBat one time only

; temp3 preloaded with 2 above if no warning state output took place
l63:	mov	OutKey,VBatChar1
		rcall	DigitOut
		mov	OutKey,VBatChar2
		rcall	DigitOut
		mov	OutKey,VBatChar3
		ldi	AddWait,70						; add another pause after each sequence of chars
		rcall	DigitOut
		dec	temp3
		brne	l63
		
		cbi	ADCSRA,ADEN						; disable ADC
		pop	r7
		pop	r6
		ret

;****************************************************************************
; Default values for EEPROM/Parameters
;****************************************************************************
; These are the default values that are copied from Flash to EEPROM if the
; EEPROM content was not programmed by the external programmer
; Note that the very first byte in EEPROM is a magic number to indicate whether
; the OSCCAL value in the second byte is set
EEPROMDefaultValues:
.DB Version, LayoutVersion					; Version, Layout Version
.DB InitKey, InitSpeed						; Key, Normal Speed
.DB InitVBat, InitCallsignSpeed			; VBat, Callsign speed
.DB InitCallsignMode, InitDelayLo		; Callsign mode, DelayLo
.DB InitDelayHi, InitDelayMode			; DelayHi, Delay mode
.DB InitTransCycle, InitTransCycPos		; Transmission cycle, Position in transmission
.DB InitConfigWithTX, Digit12sLast		; With TX/Only LED, Last Digit of 12s duration
.DB Digit12sMiddle, Digit12sFirst		; Middle Digit of 12s duration, First Digit of 12s duration
.DB ExtendISR12s, ExtendISR1min 			; ISR cycle duration extension for 12s, 1min
.DB ExtendISR5min, DigitPauseLast		; ISR cycle duration extension for 5min, Pause digits
.DB DigitPauseMiddle, DigitPauseFirst	; Pause digits
.DB DigitVBat1Last, DigitVBat1Middle	; VBat1 digits
.DB 1 , DigitVBat2Last						; VBat1/VBat2 digits
.DB DigitVBat2Middle, 1 					; VBat2 digits
;Callsign
#ifndef TEST
.DB 255, 255									; termination (255) = no call
#else
.DB CK('d'),CK('l')							; TEST
.DB DK('3'),CK('s')							; TEST
.DB CK('d'),CK('o')							; TEST
.DB 254, 254									; TEST: pause, pause, !DEP_7
.DB 255, 255									; termination (255)
#endif

;****************************************************************************
; Configuration
;****************************************************************************
; data structure that defines the configuration
; startvalue, endvalue, follow-up configuration (0=none), configoffset (in EEPROM and RAM)
; if configoffset = 0 then start with 'startvalue' instead using the EEPROM value pointed to by configoffset
ConfigDataSelection:
; 0: Selection of configuration item
.DB 1 , 18										; don't allow to configure lower digits
.DB 0 , 0


; 1: Key selection
.DB 0 , CharsEnd
.DB 0 , EE_Key


; 2: Speed selection
.DB 0 , 9
.DB 0 , EE_Speed


; 3: VBat configuration
.DB 0 , 3
.DB 0 , EE_VBat
; 0: no VBat measurement
; 1: VBat measurement and output of value twice
; 2: VBat measurement and compare with VBat1 (= battery)
; 3: VBat measurement and compare with VBat2 (= rechargeable battery)


; 4: Callsign configuration !! Placeholder config. data !!		!DEP_4!
; Callsign configuration is done separately
.DB 0 , 0
.DB 0 , 0

; 5: Callsign speed selection 
.DB 0 , 9
.DB 0 , EE_CallsignSpeed

; 6: Callsign transmission mode
.DB 0 , 5
.DB 0 , EE_CallsignMode
; 0: no callsign transmission
; 1: transmit only callsign
; 2: transmit callsign every 5min				; !DEP_1!
; 3: transmit callsign every 10min
; 4: transmit callsign every 15min
; 5: transmit callsign every 20min


; 7: Delay configuration !! Placeholder config. data !!		!DEP_5!
; Delay configuration is done separately
.DB 0 , 0
.DB 0 , 0

; 8: Delay mode
.DB 0 , 2
.DB 0 , EE_DelayMode
; 0: no Delay applied
; 1: apply Delay if pin pulled low
; 2: always apply Delay


; 9: Transmission cycle
.DB 0 , 4
.DB 0 , EE_TransmissionCycle
; 0: continuous transmission
; 1: 1min interval (sprint)
; 2: 5min interval
; 3: 1min interval (sprint), wait for trigger
; 4: 5min interval, wait for trigger

; 10: Transmission cycle position
.DB 1 , 5
.DB 0 , EE_TransCyclePos


; 11: TxIndication configuration
.DB 0 , 1
.DB 0 , EE_TxIndication
; 0: only LED indication
; 1: LED and TX indication


; 12: Pause first digit configuration
.DB 0, 1												; !DEP_6!
.DB 19 , EE_DigitPauseFirst					; continue with middle digit (conf. item 19)


; 13: VBat1 middle digit configuration
.DB 0 , 5											; limit VBat1 between 1,00 and 1,59V
.DB 21 , EE_DigitVBat1Middle					; continue with last digit (conf. item 21)

; 14: VBat2 middle digit configuration
.DB 0 , 5											; limit VBat2 between 1,00 and 1,59V
.DB 22 , EE_DigitVBat2Middle					; continue with last digit (conf. item 22)


; 15: 12s duration first digit configuration
.DB 6 , 8
.DB 23 , EE_Digit12First						; continue with middle digit (conf. item 23)


; 16: 12s duration extension
.DB 0 , 4
.DB 0 , EE_ExtendISR12s

; 17: 1min duration extension
.DB 0 , 4
.DB 0 , EE_ExtendISR1min

; 18: 5min duration extension
.DB 0 , 5
.DB 0 , EE_ExtendISR5min

;------------ not directly selectable, entered from config. above 
; 19: Pause duration middle digit configuration
.DB 0 , 9
.DB 20 , EE_DigitPauseMiddle					; continue with last digit (conf. item 20)

; 20: Pause duration last digit configuration
.DB 0 , 9
.DB 0 , EE_DigitPauseLast

; 21: VBat1 last digit configuration
.DB 0 , 9											; limit VBat1 between 1,00 and 1,59V
.DB 0 , EE_DigitVBat1Last

; 22: VBat2 last digit configuration
.DB 0 , 9											; limit VBat2 between 1,00 and 1,59V
.DB 0 , EE_DigitVBat2Last

; 23: 12s duration middle digit configuration
.DB 0 , 9
.DB 24 , EE_Digit12Middle						; continue with last digit (conf. item 24)

; 24: 12s duration last digit configuration
.DB 0 , 9
.DB 0 , EE_Digit12Last

;****************************************************************************
; Key patterns
;****************************************************************************
;normal chars
PatternStart:
.DB 0b00000101 , 0b00011000			; a, b		0
.DB 0b00011010 , 0b00001100			; c, d		2
.DB 0b00000010 , 0b00010010			; e, f		4
.DB 0b00001110 , 0b00010000			; g, h		6
.DB 0b00000100 , 0b00010111			; i, j		8
.DB 0b00001101 , 0b00010100			; k, l		10
.DB 0b00000111 , 0b00000110			; m, n		12
.DB 0b00001111 , 0b00010110			; o, p		14
.DB 0b00011101 , 0b00001010			; q, r		16
.DB 0b00001000 , 0b00000011			; s, t		18
.DB 0b00001001 , 0b00010001			; u, v		20
.DB 0b00001011 , 0b00011001			; w, x		22
.DB 0b00011011 , 0b00011100			; y, z		24
.DB 0b00111111 , 0b00101111			; 0, 1		26
.DB 0b00100111 , 0b00100011			; 2, 3		28
.DB 0b00100001 , 0b00100000			; 4, 5		30
.DB 0b00110000 , 0b00111000			; 6, 7		32
.DB 0b00111100 , 0b00111110			; 8, 9		34
.DB 0b00110001 , 0						; =			36

;special chars
SpecCharsStart:
Daaah:
.DB 12 , 0b01111111
.DB 0b00000000 , 0b00000000
Correction:
.DB 20 , 0b10101010
.DB 0b10101010 , 0b00000000

;strings
StringCharsStart:
.DB 3, CK('m')								; len=3 , m
.DB CK('o'),CK('e')						; o e

.DB 3, CK('m')								; len=3 , m
.DB CK('o'),CK('i')						; o i

.DB 3, CK('m')								; len=3 , m
.DB CK('o'),CK('s')						; o s

.DB 3, CK('m')								; len=3 , m
.DB CK('o'),CK('h')						; o h

.DB 3, CK('m')								; len=3 , m
.DB CK('o'),DK('5')						; o 5

.DB 2, CK('m')								; len=2 , m
.DB CK('o'), 4								; o , len=4
.DB CK('m'),CK('o')						; m , o
.DB 4 , (Daaah-PatternStart)*2		; e , daaah

.DB 4, CK('m')								; len=4 , m
.DB CK('o'),CK('i')						; o i
.DB (Daaah-PatternStart)*2,0			; daaah, <filler byte (is ignored)>

.DB 4, CK('m')								; len=4 , m
.DB CK('o'),CK('s')						; o s
.DB (Daaah-PatternStart)*2,0			; daaah, <filler byte (is ignored)>

.DB 4, CK('m')								; len=4 , m
.DB CK('o'),CK('h')						; o h
.DB (Daaah-PatternStart)*2,0			; daaah, <filler byte (is ignored)>

.DB 4, CK('m')								; len=4 , m
.DB CK('o'),DK('5')						; o 5
.DB (Daaah-PatternStart)*2,0			; daaah, <filler byte (is ignored)>

.DB 3, CK('m')								; len=3 , m
.DB CK('o'),(Daaah-PatternStart)*2	; o daaah

StringCharsEnd:
;everything below the StringCharsEnd label is not selectable as key during normal output
;
;version string
.DB 4, CK('v')								; len=4 , v
#ifdef __ATtiny45__
.DB DK('5'),DK('2')						; 5 2
#endif
#ifdef __ATtiny25__
.DB DK('5'),DK('2')						; 5 2
#endif
.DB (Daaah-PatternStart)*2 , 0		; daaah, <filler byte (is ignored)>

;for callsign configuration
SDahString:
.DB 2, CK('s')								; len=2 , s
.DB (Daaah-PatternStart)*2 , 0		; daaah, <filler byte (is ignored)>

PDahString:
.DB 2, CK('p')								; len=2 , p
.DB (Daaah-PatternStart)*2 , 0		; daaah, <filler byte (is ignored)>


;****************************************************************************
; Changelog
;****************************************************************************
;
;   Date   Version   Change
;----------------------------------------------------------------------------
; 23.01.05   1.0     Release
; 07.08.05   1.1     - The TX is switched off during configuration since the
;                      reading of the config. switch is not always immune against EMC
;                    - The PB0 pull-up was always enabled. Since the config.
;                      switch is now only checked when the TX is off, the
;                      pull-up is disabled after the initial checks for configuration
;                    - added some comments
;                    - modified some comments
; 26.05.06           The software is ported to the ATtiny13
; 01.11.07   2.0     Release
; 01.11.11   3.0     The software is ported to the ATtiny25
; 27.03.15   3.1     The keying speed is made configurable. The duration of a dot is 
;                    lengthened to better match 50 and 70 BpM
;                    During key configuration: the code option to store a key when 
;                    pressing the switch instead of proceeding to the next key was removed
; 05.12.15   3.2     Measuring of battery voltage
;                    Because of battery voltage at PB2 the expert mode is now switched off/on
;                    by pulling MISO (PB1) to ground
; 21.04.16   4.2     The software is ported to the ATtiny45
;                    It is configurable whether the software compiles for an ATtiny25 or
;                    ATtiny45. If ATtiny25 is selected the software identifies itself
;                    as Version 3.2 otherwise as Version 4.2
; 18.06.16   5.0     Major redesign:
;                    The configuration is now done completely different (in two steps)
;                    Expert mode removed
;                    Some code refactoring and optimization
;                    Possibility to clock by the 3,579545MHz clock out of the 74HC00 osc. gate
;                    Moved from GPL v2 to GPL v3 license
;                    New features:
;                    callsign configuration, callsign speed configuration, callsign transmission
;                    interval configuration, interval position
;                    configuration output by TX or only LED, VBat measurement configurable
;                    number of ISR ticks for 12s duration can be configured
;                    VBat (configurable) warning level
;							configurable pause between chars/strings in normal output
;							PRE-RELEASE
; 03.09.16   5.0     Don't copy callsign from EEPROM to RAM if callsign output is based on the data 
;                    in EEPROM anyway
;                    Different keying speeds for 1st and 2nd stage of configuration
;                    Trigger Watchdog if only callsign output is configured but no callsign
; 03.05.17   5.1     If the Config switch is pressed longer than 2.1s during key configuration then 
;                    the next key to be put out is MOE
;                    If the Config switch is pressed longer than 2.1s during callsign configuration 
;                    then the next key to be put out is 0
; 19.05.20   5.2     Default for Battery voltage measurement is now 2
;                    QUARTZ mode can be selected through command line parameter
